@@ -17,51 +17,58 @@
 #
 
 from argparse import Namespace
+from enum import IntEnum
 from pathlib import Path
 
 import httpx
 
 from pontos import changelog
+from pontos.git import Git
 from pontos.github.api import GitHubRESTApi
-from pontos.helper import shell_cmd_runner
 from pontos.terminal import Terminal
 
 from .helper import (
     commit_files,
     find_signing_key,
     get_current_version,
+    get_git_repository_name,
     get_next_dev_version,
-    get_project_name,
     update_version,
 )
 
 RELEASE_TEXT_FILE = ".release.md"
 
 
+class ReleaseReturnValue(IntEnum):
+    SUCCESS = 0
+    TOKEN_MISSING = 1
+    NO_RELEASE_VERSION = 2
+    CREATE_RELEASE_ERROR = 3
+    UPDATE_VERSION_ERROR = 4
+
+
 def release(
     terminal: Terminal,
     args: Namespace,
     *,
     token: str,
     **_kwargs,
-) -> bool:
+) -> IntEnum:
     if not token:
         terminal.error(
             "Token is missing. The GitHub token is required to create a "
             "release."
         )
-        return False
+        return ReleaseReturnValue.TOKEN_MISSING
 
     project: str = (
-        args.project
-        if args.project is not None
-        else get_project_name(shell_cmd_runner)
+        args.project if args.project is not None else get_git_repository_name()
     )
     space: str = args.space
     git_signing_key: str = (
         args.git_signing_key
         if args.git_signing_key is not None
-        else find_signing_key(terminal, shell_cmd_runner)
+        else find_signing_key(terminal)
     )
     git_remote_name: str = (
         args.git_remote_name if args.git_remote_name is not None else ""
@@ -72,6 +79,10 @@ def release(
         if args.release_version is not None
         else get_current_version(terminal)
     )
+    if not release_version:
+        terminal.error("No release version available.")
+        return ReleaseReturnValue.NO_RELEASE_VERSION
+
     next_version: str = (
         args.next_version
         if args.next_version is not None
@@ -80,7 +91,8 @@ def release(
 
     terminal.info("Pushing changes")
 
-    shell_cmd_runner(f"git push --follow-tags {git_remote_name}")
+    git = Git()
+    git.push(follow_tags=True, remote=git_remote_name)
 
     terminal.info(f"Creating release for v{release_version}")
     changelog_text: str = Path(RELEASE_TEXT_FILE).read_text(encoding="utf-8")
@@ -99,7 +111,7 @@ def release(
         )
     except httpx.HTTPError as e:
         terminal.error(str(e))
-        return False
+        return ReleaseReturnValue.CREATE_RELEASE_ERROR
 
     Path(RELEASE_TEXT_FILE).unlink()
 
@@ -133,17 +145,17 @@ def release(
 
     executed, filename = update_version(terminal, next_version, develop=True)
     if not executed:
-        return False
+        return ReleaseReturnValue.UPDATE_VERSION_ERROR
 
     commit_files(
+        git,
         filename=filename,
         commit_msg=commit_msg,
-        shell_cmd_runner=shell_cmd_runner,
         git_signing_key=git_signing_key,
         changelog=changelog_bool,
     )
 
     # pushing the new tag
-    shell_cmd_runner(f"git push --follow-tags {git_remote_name}")
+    git.push(follow_tags=True, remote=git_remote_name)
 
-    return True
+    return ReleaseReturnValue.SUCCESS
